//
// LocalFileStoreLoader.cs: loads backends from the local filestore
//
// Authors:
//   Nat Friedman <nat@nat.org>
//   Joe Shaw <joe@assbarn.com>
//   Miguel de Icaza <miguel@ximian.com>
//   Richard Taylor - seperation into backendloader class.
//


using System;
using System.IO;
using System.Xml;
using System.Collections;
using System.Reflection;
using System.Threading;


[assembly:Dashboard.BackendLoaderFactory ("Dashboard.LocalFileStoreLoader")]

namespace Dashboard {

	class LocalFileStoreLoader : BackendLoaderSimple {


		// Search the backends directory for DLLs that are
		// Dashboard backends and start them up.
		public override void LoadBackends ()
		{
			//backends = new ArrayList ();

			string backends_dir = Environment.GetEnvironmentVariable ("DASHBOARD_BACKENDS_PATH");
			if (backends_dir == null || backends_dir == "")
				backends_dir = "../backends";

			string [] files = Directory.GetFiles (backends_dir, "*.dll");

			foreach (string file in files) {
				try {

					Assembly a = Assembly.LoadFrom (file);

					BackendFactoryAttribute attr = (BackendFactoryAttribute) a.GetCustomAttributes (typeof (BackendFactoryAttribute), false)[0];
					
					if (attr == null)
						continue;

					Console.WriteLine ("Loading backend: " + file);

					Backend backend = (Backend) Activator.CreateInstance (a.GetType (attr.FactoryClass));
	
					ThreadPool.QueueUserWorkItem (new WaitCallback (this.LaunchBackend), backend);
				} catch {
					//
					// Ignore an error if the file is not really an assembly
					//
				}
			}
		}

		// In a separate thread
		private void LaunchBackend (object o)
		{
			if (o == null)
				return;

			try {
				Backend b = (Backend) o;
				
				if (b.Startup ()) {
					lock (this.parent.backends) {
						Console.WriteLine ("Before call: {0}\n" + b.GetType ().ToString ());
						this.parent.backends.Add (b);
					}
				} else {
					Console.WriteLine ("{0} NOT loaded", b.Name);
				}
			} catch (Exception e) {
				Console.WriteLine ("Received exception while loading Backend:");
				Console.WriteLine ("   Backend type: {0}\n" + o.GetType ().ToString ());
				Console.WriteLine ("Exception is:\n{0}", e);
			}
		}

	} // Class

} // Namespace
